/*
 *************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *  
 *************************************************************************
 */

package org.eclipse.birt.data.engine.aggregation.rank;

import org.eclipse.birt.data.engine.api.aggregation.Accumulator;
import org.eclipse.birt.data.engine.api.aggregation.IAggregation;
import org.eclipse.birt.data.engine.api.aggregation.IBuildInAggregation;
import org.eclipse.birt.data.engine.core.DataException;
import org.eclipse.birt.data.engine.i18n.ResourceConstants;

/**
 * Implements the built-in Total.Quartile aggregation.
 */
public class TotalQuartile implements IAggregation
{

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.data.engine.aggregation.Aggregation#getName()
	 */
	public String getName( )
	{
		return IBuildInAggregation.TOTAL_QUARTILE_FUNC;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.data.engine.aggregation.Aggregation#getType()
	 */
	public int getType( )
	{
		return SUMMARY_AGGR;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.data.engine.aggregation.Aggregation#getParameterDefn()
	 */
	public boolean[] getParameterDefn( )
	{
		return new boolean[]{
				true, false
		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.data.engine.aggregation.Aggregation#newAccumulator()
	 */
	public Accumulator newAccumulator( )
	{
		return new MyAccumulator( );
	}

	/**
	 * 
	 *
	 */
	private class MyAccumulator extends PercentileAccumulator
	{
		/*
		 * (non-Javadoc)
		 * @see org.eclipse.birt.data.engine.aggregation.rank.PercentileAccumulator#getPctValue(java.lang.Double)
		 */
		protected double getPctValue( Double d ) throws DataException
		{
			validatePctValue( d );
			int quar = d.intValue( );
			double result = 0;
			if ( quar == 0 )
				result = 0;
			else if ( quar == 1 )
				result = 0.25;
			else if ( quar == 2 )
				result = 0.5;
			else if ( quar == 3 )
				result = 0.75;
			else if ( quar == 4 )
				result = 1;

			return result;
		}

		/**
		 * 
		 * @param d
		 * @throws DataException
		 */
		private void validatePctValue( Double d ) throws DataException
		{
			if ( d == null
					|| d.isNaN( ) || d.doubleValue( ) < 0
					|| d.doubleValue( ) > 4 )
				throw new DataException( ResourceConstants.INVALID_QUARTILE_ARGUMENT );
		}
	}
}
